package xyz.imaf6971.volgait.service.impl;

import org.springframework.stereotype.Service;
import xyz.imaf6971.volgait.dto.AdvertisementDto;
import xyz.imaf6971.volgait.dto.PublishAdvertisementDto;
import xyz.imaf6971.volgait.exceptions.AdvertisementNotFoundException;
import xyz.imaf6971.volgait.models.Advertisement;
import xyz.imaf6971.volgait.repository.AdvertisementRepository;
import xyz.imaf6971.volgait.service.AdvertisementService;
import xyz.imaf6971.volgait.service.AdvertisementSubcategoryService;
import xyz.imaf6971.volgait.service.AdvertisementTypeService;
import xyz.imaf6971.volgait.service.UserService;

import javax.transaction.Transactional;
import java.time.LocalDateTime;
import java.util.List;

@Service
public class AdvertisementServiceImpl implements AdvertisementService {

    private final AdvertisementRepository advertisementRepository;

    private final AdvertisementTypeService advertisementTypeService;

    private final AdvertisementSubcategoryService advertisementSubcategoryService;

    private final UserService userService;

    public AdvertisementServiceImpl(AdvertisementRepository advertisementRepository, AdvertisementTypeService advertisementTypeService, AdvertisementSubcategoryService advertisementSubcategoryService, UserService userService) {
        this.advertisementRepository = advertisementRepository;
        this.advertisementTypeService = advertisementTypeService;
        this.advertisementSubcategoryService = advertisementSubcategoryService;
        this.userService = userService;
    }

    @Override
    @Transactional
    public Long publish(PublishAdvertisementDto publishDto) {
        var advertisement = new Advertisement();
        setInfoFromDto(advertisement, publishDto);
        setType(advertisement, publishDto.typeId());
        setSubcategory(advertisement, publishDto.subcategoryId());
        setAuthor(advertisement);
        advertisement = advertisementRepository.save(advertisement);
        return advertisement.getId();
    }

    private void setAuthor(Advertisement advertisement) {
        var author = userService.getCurrentUser();
        advertisement.setAuthor(author);
    }

    private void setSubcategory(Advertisement advertisement, Long subcategoryId) {
        var subcategory = advertisementSubcategoryService.getSubcategoryById(subcategoryId);
        advertisement.setSubcategory(subcategory);
    }

    private void setType(Advertisement advertisement, Long typeId) {
        var type = advertisementTypeService.getTypeById(typeId);
        advertisement.setType(type);
    }

    private void setInfoFromDto(Advertisement advertisement, PublishAdvertisementDto publishDto) {
        advertisement.setTitle(publishDto.title());
        advertisement.setText(publishDto.text());
        advertisement.setCreationDate(LocalDateTime.now());
    }

    @Override
    public List<AdvertisementDto> listAll() {
        return advertisementRepository.findAll()
                .stream().map(this::toDto)
                .toList();
    }

    @Override
    public AdvertisementDto getAdvertisementById(Long advertisementId) {
        var adv = getById(advertisementId);
        return toDto(adv);
    }

    private AdvertisementDto toDto(Advertisement advertisement) {
        return new AdvertisementDto(
                advertisement.getId(),
                advertisement.getTitle(),
                advertisement.getText(),
                advertisement.getCreationDate(),
                advertisement.getSubcategory().getTitle(),
                advertisement.getType().getTitle());
    }

    private Advertisement getById(Long advertisementId) {
        return advertisementRepository.findById(advertisementId)
                .orElseThrow(() -> new AdvertisementNotFoundException("Advertisement with id " + advertisementId + " not found"));
    }

    @Override
    @Transactional
    public void changeById(PublishAdvertisementDto changes, Long advertisementId) {
        var oldAdvertisement = getById(advertisementId);
        changeTitleIfNotNull(oldAdvertisement, changes.title());
        changeTextIfNotNull(oldAdvertisement, changes.text());
        changeCategoryIfNotNull(oldAdvertisement, changes.subcategoryId());
        changeTypeIfNotNull(oldAdvertisement, changes.typeId());
        advertisementRepository.save(oldAdvertisement);
    }

    @Override
    public void deleteAdvertisementById(Long advertisementId) {
        advertisementRepository.deleteById(advertisementId);
    }

    private void changeTypeIfNotNull(Advertisement oldAdvertisement, Long typeId) {
        if (typeId != null) {
            var type = advertisementTypeService.getTypeById(typeId);
            oldAdvertisement.setType(type);
        }
    }

    private void changeCategoryIfNotNull(Advertisement oldAdvertisement, Long subcategoryId) {
        if (subcategoryId != null) {
            var subcategory = advertisementSubcategoryService.getSubcategoryById(subcategoryId);
            oldAdvertisement.setSubcategory(subcategory);
        }
    }

    private void changeTextIfNotNull(Advertisement oldAdvertisement, String newText) {
        if (newText != null) {
            oldAdvertisement.setText(newText);
        }
    }

    private void changeTitleIfNotNull(Advertisement oldAdvertisement, String newTitle) {
        if (newTitle != null) {
            oldAdvertisement.setTitle(newTitle);
        }
    }

}
